@@ -2,7 +2,7 @@
 """DB-SIG compliant module for communicating with MS SQL servers"""
 
 __author__ = 'Mikhail Denisenko <denisenkom@gmail.com>'
-__version__ = '0.3.0'
+__version__ = '0.4.0'
 
 import logging
 import decimal
@@ -163,13 +163,6 @@ class Connection(object):
         """
         return self._autocommit
 
-    @property
-    def connected(self):
-        """
-        True if the connection to a database is open.
-        """
-        return self._connected
-
     @property
     def rows_affected(self):
         """
@@ -190,6 +183,20 @@ class Connection(object):
     def chunk_handler_set(self, value):
         self.tds_socket.chunk_handler = value
 
+    @property
+    def tds_version(self):
+        '''
+        Returns version of tds protocol that is being used by this connection
+        '''
+        return self.tds_socket.tds_version
+
+    @property
+    def product_version(self):
+        '''
+        Returns version of the server
+        '''
+        return self.tds_socket.product_version
+
     def _get_connection(self):
         if self.tds_socket.is_dead():
             # clear transaction
@@ -223,7 +230,6 @@ class Connection(object):
             as_dict, encryption_level, login_timeout, timeout):
         self._autocommit = False
         logger.debug("Connection.__init__()")
-        self._connected = 0
         self._charset = ''
         self.last_msg_no = 0
         self.last_msg_severity = 0
@@ -232,6 +238,8 @@ class Connection(object):
         self.last_msg_srv = ''
         self.last_msg_proc = ''
         self._as_dict = as_dict
+        self._state = DB_RES_NO_MORE_RESULTS
+        self.tds_socket = None
 
         # support MS methods of connecting locally
         instance = ""
@@ -241,8 +249,6 @@ class Connection(object):
         if server in (".", "(local)"):
             server = "localhost"
 
-        server = server + "\\" + instance if instance else server
-
         self._login = login = tds_alloc_login(1)
         # set default values for loginrec
         login.library = "Python TDS Library"
@@ -250,25 +256,14 @@ class Connection(object):
         appname = appname or "pytds"
 
         login.encryption_level = encryption_level
-        login.user_name = user
-        login.password = password
+        login.user_name = user or ''
+        login.password = password or ''
         login.app = appname
+        login.port = port
         if tds_version:
-            login.tds_version = _tds_ver_str_to_constant(tds_version)
+            login.tds_version = tds_version
         login.database = database
 
-        # override the HOST to be the portion without the server, otherwise
-        # FreeTDS chokes when server still has the port definition.
-        # BUT, a patch on the mailing list fixes the need for this.  I am
-        # leaving it here just to remind us how to fix the problem if the bug
-        # doesn't get fixed for a while.  But if it does get fixed, this code
-        # can be deleted.
-        # patch: http://lists.ibiblio.org/pipermail/freetds/2011q2/026997.html
-        #if ':' in server:
-        #    os.environ['TDSHOST'] = server.split(':', 1)[0]
-        #else:
-        #    os.environ['TDSHOST'] = server
-
         # Set the character set name
         if charset:
             _charset = charset
@@ -276,16 +271,14 @@ class Connection(object):
             login.charset = self._charset
 
         # Connect to the server
-        tds_set_server(login, server)
+        login.server_name = server
+        login.instance_name = instance
         ctx = tds_alloc_context()
         ctx.msg_handler = self._msg_handler
         ctx.err_handler = self._err_handler
         ctx.int_handler = self._int_handler
         self.tds_socket = tds_alloc_socket(ctx, 512)
         self.tds_socket.chunk_handler = MemoryChunkedHandler()
-        self.tds_socket.env_chg_func = self._db_env_chg
-        self._curdb = ''
-        self._servcharset = ''
         login.option_flag2 &= ~0x02 # we're not an ODBC driver
         tds_fix_login(login) # initialize from Environment variables
 
@@ -365,18 +358,6 @@ class Connection(object):
             self.last_msg_srv = msg['server']
             self.last_msg_proc = msg['proc_name']
 
-    def _db_env_chg(self, tds, type, oldval, newval):
-        assert oldval is not None and newval is not None
-        if oldval == '\x01':
-            oldval = "(0x1)"
-
-        logger.debug("db_env_chg(%d, %s, %s)", type, oldval, newval)
-
-        if type == TDS_ENV_DATABASE:
-            self._curdb = newval
-        elif type == TDS_ENV_CHARSET:
-            self._servcharset = newval
-
     def __del__(self):
         logger.debug("MSSQLConnection.__del__()")
         self.close()
@@ -392,8 +373,10 @@ class Connection(object):
         logger.debug("MSSQLConnection.cancel()")
         self.clr_err()
 
-        tds_send_cancel(self.tds_socket)
-        tds_process_cancel(self.tds_socket)
+        tds = self.tds_socket
+        if not tds.is_dead():
+            tds_send_cancel(tds)
+            tds_process_cancel(tds)
 
     def close(self):
         """
@@ -404,21 +387,12 @@ class Connection(object):
         this case.
         """
         logger.debug("MSSQLConnection.close()")
-        if self == None:
-            return None
-
-        if not self._connected:
-            return None
-
         self.clr_err()
-
         tds = self.tds_socket
-        if tds:
+        if tds is not None:
+            tds_close_socket(tds)
             tds_free_socket(tds)
 
-        self._connected = 0
-
-
     def select_db(self, dbname):
         """
         select_db(dbname) -- Select the current database.
@@ -760,7 +734,7 @@ class Cursor(object):
             self._conn._nextrow()
             check_cancel_and_raise(self._conn)
         self._get_results()
-        return self._conn._state != DB_RES_NO_MORE_RESULTS
+        return None if self._conn._state == DB_RES_NO_MORE_RESULTS else True
 
     def _get_results(self):
         if not self._results:
@@ -836,7 +810,7 @@ class Cursor(object):
 
 def connect(server='.', database='', user='', password='', timeout=0,
         login_timeout=60, charset=None, as_dict=False,
-        host='', appname=None, port='1433', tds_version='',
+        host='', appname=None, port=None, tds_version=TDS74,
         encryption_level=TDS_ENCRYPTION_OFF):
     """
     Constructor for creating a connection to the database. Returns a
@@ -885,26 +859,6 @@ def connect(server='.', database='', user='', password='', timeout=0,
 
     return conn
 
-
-def _tds_ver_str_to_constant(verstr):
-    """
-        http://www.freetds.org/userguide/choosingtdsprotocol.htm
-    """
-    if verstr == u'4.2':
-        return 0x402
-    elif verstr == u'7.0':
-        return 0x700
-    elif verstr == u'7.1':
-        return 0x701
-    elif verstr == u'7.2':
-        return 0x702
-    elif verstr == '7.3':
-        return 0x703
-    #elif verstr == u'8.0':
-    #    return 0x800
-    else:
-        raise MSSQLException('unrecognized tds version: %s' % verstr)
-
 def get_last_msg_str(conn):
     return conn.last_msg_str
 
