@@ -148,7 +148,7 @@ def tds_submit_rpc(tds, rpc_name, params=(), recompile=False):
             # send it
             tds_query_flush_packet(tds)
             # emulate it for TDS4.x, send RPC for mssql
-            if tds.tds_version < 0x500:
+            if not IS_TDS5_PLUS(tds):
                 return tds_send_emulated_rpc(tds, rpc_name, params)
     except:
         tds_set_state(tds, TDS_IDLE)
@@ -552,7 +552,7 @@ def tds_submit_begin_tran(tds):
         tds_submit_query(tds, "BEGIN TRANSACTION")
 
 def tds_submit_rollback(tds, cont):
-    logger.debug('tds_submit_rollback(%s)', cont)
+    logger.debug('tds_submit_rollback(%s, %s)', id(tds), cont)
     if IS_TDS72(tds):
         if tds_set_state(tds, TDS_QUERYING) != TDS_QUERYING:
             raise Exception('TDS_FAIL')
