@@ -1,5 +1,5 @@
 import logging
-from BitVector import BitVector
+import traceback
 from read import *
 from tdsproto import *
 from iconv import *
@@ -125,20 +125,20 @@ def tds_process_default_tokens(tds, marker):
     elif marker == TDS_CURINFO_TOKEN:
         return tds_process_cursor_tokens(tds)
     elif marker in (TDS5_DYNAMIC_TOKEN, TDS_LOGINACK_TOKEN, TDS_ORDERBY_TOKEN, TDS_CONTROL_TOKEN):
-        logger.warning("Eating %s token", tds_token_name(marker))
+        logger.debug("Eating %s token", tds_token_name(marker))
         tds_skip_n(tds, tds_get_smallint(tds))
     elif marker == TDS_TABNAME_TOKEN: # used for FOR BROWSE query
         return tds_process_tabname(tds)
     elif marker == TDS_COLINFO_TOKEN:
         return tds_process_colinfo(tds, None, 0)
     elif marker == TDS_ORDERBY2_TOKEN:
-        logger.warning("Eating %s token", tds_token_name(marker))
+        logger.debug("Eating %s token", tds_token_name(marker))
         tds_skip_n(tds, tds_get_int(tds))
     elif marker == TDS_NBC_ROW_TOKEN:
         return tds_process_nbcrow(tds)
     else:
         tds_close_socket(tds)
-        logger.error('Unknown marker: {0}({0:x})'.format(marker))
+        logger.error('Unknown marker: {0}({0:x}) {1}'.format(marker, ''.join(traceback.format_stack())))
         raise Exception('TDSEBTOK')
 
 #
@@ -170,10 +170,9 @@ def tds_process_nbcrow(tds):
 
     # reading bitarray for nulls, 1 represent null values for
     # corresponding fields
-    nbc = tds_get_n(tds, len(info.columns) + 7 / 8)
-    nbc = BitVector(bitstring=(ord(b) for b in nbc))
+    nbc = tds_get_n(tds, (len(info.columns) + 7) / 8)
     for i, curcol in enumerate(info.columns):
-        if nbc[i]:
+        if ord(nbc[i/8]) & (1 << i%8):
             curcol.value = None
         else:
             curcol.funcs.get_data(tds, curcol)
@@ -201,7 +200,7 @@ def tds_process_end(tds, marker):
         tds.res_info.more_results = more_results
         if not tds.current_results:
             tds.current_results = tds.res_info
-    rows_affected = tds_get_int8(tds) if tds.tds_version >= 0x702 else tds_get_int(tds)
+    rows_affected = tds_get_int8(tds) if IS_TDS72_PLUS(tds) else tds_get_int(tds)
     logger.debug('\t\trows_affected = {0}'.format(rows_affected))
     if was_cancelled or (not more_results and not tds.in_cancel):
         logger.debug('tds_process_end() state set to TDS_IDLE')
@@ -232,33 +231,17 @@ def tds_process_env_chg(tds):
         logger.debug("tds.collation now {0}".format(tds.collation));
         # discard old one
         tds_get_n(tds, tds_get_byte(tds))
-        return TDS_SUCCESS
     elif type == TDS_ENV_BEGINTRANS:
         size = tds_get_byte(tds)
         tds.tds72_transaction = tds_get_n(tds, 8)
         tds_get_n(tds, tds_get_byte(tds))
-        return TDS_SUCCESS
     elif type == TDS_ENV_COMMITTRANS or type == TDS_ENV_ROLLBACKTRANS:
         tds.tds72_transaction = None
         tds_get_n(tds, tds_get_byte(tds))
         tds_get_n(tds, tds_get_byte(tds))
-        return TDS_SUCCESS
-    # discard byte values, not still supported
-    # TODO support them
-    elif IS_TDS71_PLUS(tds) and type > TDS_ENV_PACKSIZE:
-        # discard new one
-        tds_get_n(tds, tds_get_byte(tds))
-        # discard old one
-        tds_get_n(tds, tds_get_byte(tds))
-        return TDS_SUCCESS
-
-    # fetch the new value
-    newval = tds_get_string(tds, tds_get_byte(tds))
-
-    # fetch the old value
-    oldval = tds_get_string(tds, tds_get_byte(tds))
-
-    if type == TDS_ENV_PACKSIZE:
+    elif type == TDS_ENV_PACKSIZE:
+        newval = tds_get_string(tds, tds_get_byte(tds))
+        oldval = tds_get_string(tds, tds_get_byte(tds))
         new_block_size = int(newval)
         if new_block_size >= 512:
             logger.info("changing block size from {0} to {1}".format(oldval, new_block_size))
@@ -269,15 +252,30 @@ def tds_process_env_chg(tds):
             # Reallocate buffer if possible (strange values from server or out of memory) use older buffer */
             tds_realloc_socket(tds, new_block_size)
     elif type == TDS_ENV_DATABASE:
+        newval = tds_get_string(tds, tds_get_byte(tds))
+        oldval = tds_get_string(tds, tds_get_byte(tds))
         tds.env.database = newval
     elif type == TDS_ENV_LANG:
+        newval = tds_get_string(tds, tds_get_byte(tds))
+        oldval = tds_get_string(tds, tds_get_byte(tds))
         tds.env.language = newval
     elif type == TDS_ENV_CHARSET:
+        newval = tds_get_string(tds, tds_get_byte(tds))
+        oldval = tds_get_string(tds, tds_get_byte(tds))
         logger.debug("server indicated charset change to \"{0}\"\n".format(newval))
         tds.env.charset = newval
         tds_srv_charset_changed(tds, newval)
-    if tds.env_chg_func:
-        tds.env_chg_func(tds, type, oldval, newval);
+    elif type == TDS_ENV_DB_MIRRORING_PARTNER:
+        newval = tds_get_string(tds, tds_get_byte(tds))
+        oldval = tds_get_string(tds, tds_get_byte(tds))
+
+    else:
+        # discard byte values, not still supported
+        # TODO support them
+        # discard new one
+        tds_get_n(tds, tds_get_byte(tds))
+        # discard old one
+        tds_get_n(tds, tds_get_byte(tds))
 
 
 def tds_process_msg(tds, marker):
@@ -312,7 +310,7 @@ def tds_process_msg(tds, marker):
         msg['server'] = tds.server_name
     # stored proc name if available
     msg['proc_name'] = tds_get_string(tds, tds_get_byte(tds))
-    msg['line_number'] = tds_get_int(tds) if tds.tds_version >= 0x702 else tds_get_smallint(tds)
+    msg['line_number'] = tds_get_int(tds) if IS_TDS72_PLUS(tds) else tds_get_smallint(tds)
     if not msg['sql_state']:
         #msg['sql_state'] = tds_alloc_lookup_sqlstate(tds, msg['msgno'])
         pass
@@ -345,6 +343,15 @@ def tds_process_msg(tds, marker):
         elif msg['msgno']:
             logger.warn(u'Msg {msgno}, Severity {severity}, State {state}, Server {server}, Line {line_number}\n{message}'.format(**msg))
 
+_SERVER_TO_CLIENT_MAPPING = {
+    0x07000000: TDS70,
+    0x07010000: TDS71,
+    0x71000001: TDS71rev1,
+    TDS72: TDS72,
+    TDS73A: TDS73A,
+    TDS73B: TDS73B,
+    TDS74: TDS74,
+    }
 
 def tds_process_login_tokens(tds):
     succeed = False
@@ -357,51 +364,43 @@ def tds_process_login_tokens(tds):
             tds.tds71rev1 = 0
             size = tds_get_smallint(tds)
             ack = tds_get_byte(tds)
-            ver['major'] = tds_get_byte(tds)
-            ver['minor'] = tds_get_byte(tds)
-            ver['tiny0'] = tds_get_byte(tds)
-            ver['tiny1'] = tds_get_byte(tds)
-            ver['reported'] = (ver['major'] << 24) | (ver['minor'] << 16) | (ver['tiny0'] << 8) | ver['tiny1']
-            if ver['reported'] == 0x07010000:
-                tds.tds71rev1 = 1
-            if ver['reported'] == 0x07000000:
+            version = tds_get_uint_be(tds)
+            ver['reported'] = version
+            tds.tds_version = _SERVER_TO_CLIENT_MAPPING[version]
+            if tds.tds_version == TDS71rev1:
+                tds.tds71rev1 = True
+            if ver['reported'] == TDS70:
                 ver['name'] = '7.0'
-            elif ver['reported'] == 0x07010000:
+            elif ver['reported'] == TDS71:
                 ver['name'] = '2000'
-            elif ver['reported'] == 0x71000001:
+            elif ver['reported'] == TDS71rev1:
                 ver['name'] = '2000 SP1'
-            elif ver['reported'] == 0x72090002:
+            elif ver['reported'] == TDS72:
                 ver['name'] = '2005'
-            elif ver['reported'] == 0x730A0003:
+            elif ver['reported'] == TDS73A:
                 ver['name'] = '2008 (no NBCROW of fSparseColumnSet)'
-            elif ver['reported'] == 0x730B0003:
+            elif ver['reported'] == TDS73B:
                 ver['name'] = '2008'
+            elif version == TDS74:
+                ver['name'] = '2012'
             else:
                 ver['name'] = 'unknown'
-            logger.debug('server reports TDS version {major:x}.{minor:x}.{tiny0:x}.{tiny1:x}'.format(**ver))
-            logger.debug('Product name for {reported:x} is {name}'.format(**ver))
+            logger.debug('server reports TDS version {0:x}'.format(version))
             # get server product name
             # ignore product name length, some servers seem to set it incorrectly
             tds_get_byte(tds)
             product_version = 0
             size -= 10
-            if ver['major'] >= 7:
+            if IS_TDS7_PLUS(tds):
                 product_version = 0x80000000
                 tds.product_name = tds_get_string(tds, size/2)
-            elif ver['major'] >= 5:
+            elif IS_TDS5_PLUS(tds):
                 tds.product_name = tds_get_string(tds, size)
             else:
                 tds.product_name = tds_get_string(tds, size)
-                if tds.product_name and tds.product_name.find('Microsoft') != -1:
-                    product_version = 0x80000000
-            product_version |= tds_get_byte(tds) << 24
-            product_version |= tds_get_byte(tds) << 16
-            product_version |= tds_get_byte(tds) << 8
-            product_version |= tds_get_byte(tds)
+            product_version = tds_get_uint_be(tds)
             # MSSQL 6.5 and 7.0 seem to return strange values for this
             # using TDS 4.2, something like 5F 06 32 FF for 6.50
-            if ver['major'] == 4 and ver['minor'] == 2 and (product_version & 0xff0000ff) == 0x5f0000ff:
-                product_version = (product_version & 0xffff00 | 0x800000) << 8
             tds_conn(tds).product_version = product_version
             logger.debug('Product version {0:x}'.format(product_version))
             # TDS 5.0 reports 5 on success 6 on failure
